/*
 * hfwload - firmware loader for hermes wavelan cards
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */
 
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "hermesfw.h"
#include "pdar.h"
#include "hfw.h"
#include "hermes_mem.h"

unsigned long pagesize=HERMES_AUX_PAGESIZE;
unsigned long pages=HERMES_AUX_PAGES;

int main(int argc, char **argv) {
	unsigned char pda[PDA_SIZE_MAX];
	struct hfw *hfw;
	int fd;
	
	if(argc<3) {
		fprintf(stderr,"usage: %s ethX <firmware.hfw>\n",argv[0]);
		return 0;
	}
	
	strcpy(pda,"/dev/orinoco/");
	strcat(pda,argv[1]);
	strcat(pda,"_mem");

	if((fd=open(pda,O_RDWR))<0) {
		fprintf(stderr,"can't open %s\n",pda);
		return -1;
	}
	
	if(!(hfw=read_hfw(argv[2]))) {
		fprintf(stderr,"error reading/parsing %s\n",argv[2]);
		return -1;
	}
	
	dump_segments(hfw->segms);
	//write_hfw_plugs(stderr,hfw->plugs);
	//exit(0);
	
	if(read_pda_from(fd,PAGE_TO_OFFSET(0x7200),pda)) {
		fprintf(stderr,"error reading PDA (PAGE 0x%04X POF 0 == OFFSET 0x%08lX) from %s\n",
						0x7200,PAGE_TO_OFFSET(0x7200),argv[1]);
		return -1;
	}
	
	//else
//		dump_pda(pda,PDA_SIZE_MAX);
	
	if(hfw_plug_records(hfw,pda)) {
		fprintf(stderr,"plugging failed - aborting\n");
		return -1;
	}
	if(!hfw->entry)
		return 0;
#ifndef TEST
	if(ioctl(fd,ORINOCO_PREPARE_RAMDL,hfw->entry)) {
		fprintf(stderr,"ORINOCO_PREPARE_RAMDL ioctl failed - aborting\n");
		fprintf(stderr,"maybe you forgot ifconfig %s down ?\n",argv[1]);
		return -1;
	}
#endif
	
	printf("hfw_upload: %d\n",hfw_upload(hfw,fd));
	
#ifndef TEST
	if(ioctl(fd,ORINOCO_FINISH_RAMDL,0)) {
		fprintf(stderr,"ORINOCO_FINISH_RAMDL ioctl failed\n");
		return -1;
	}
#endif

	return 0;
}
