/*
 * functions for binfw2hfw - hermes wavelan cards firmware extracter
 *
 * Copyright (c) 2003, Benedikt 'Hunz' Heinz <hermesap@hunz.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. See README and COPYING for
 * more details.
 */
 
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include "hermesfw.h"
#include "bin2hfw.h"
#include "hfw.h"

int check_segments(struct segmrecord *segms, struct content *cnts) {
	int count=0;
	
	while(1) {
		fprintf(stderr,"%2d: OF %8lX LEN %4X / OF %8lX LEN %8lX UNK %8lX\n",
						count, cnts[count].offset, cnts[count].length,
						segms[count].offset, segms[count].length, segms[count].unknown);
		if( (segms[count].offset != cnts[count].offset) /* || (segms[count].length != cnts[count].length)*/)
			return -1;
		else if (!(segms[count].length))
			break;
		
		count++;
	}
	return 0;
}

int read_bin_content(int fd, struct content *cnt) {
	int count, count2;
	unsigned char *ptr;
	
	if(read(fd,cnt,6)!=6)
		return -1;
	/*
	if(src_endian != __BYTE_ORDER) {
		cnt->offset=l_swap(cnt->offset);
		cnt->length=s_swap(cnt->length);
	}
	*/
	//fprintf(stderr,"%08X %04X\n",cnt->offset,cnt->length);
	if((cnt->offset > 0x800000) || (cnt->length&1) || (!cnt->length) || ((cnt->offset+cnt->length)>0x800000)) {
		cnt->length=0;
		return -1;
	}
	if(!(cnt->data=malloc(cnt->length))) {
		cnt->length=0;
		return -1;
	}
	count2=cnt->length;
	ptr=cnt->data;
	while(count2) {
		if(count2>512)
			count=512;
		else
			count=count2;
		if(read(fd,ptr,count)!=count) {
			free(cnt->data);
			cnt->length=0;
			return -1;
		}
		count2-=count;
		ptr+=count;
	}
	return 0;
}

struct segmrecord *read_bin_segms(int fd) {
	struct segmrecord *rrecs, *recs=NULL;
	int count=0;	
	
	while(1) {
		if(!(rrecs=realloc( recs, sizeof(struct segmrecord) * (count+1)))) {
			free(recs);
			return NULL;
		}
		else
			recs=rrecs;
		if(read(fd,recs+count, sizeof(struct segmrecord)) != sizeof(struct segmrecord))
			break;
		if(src_endian != __BYTE_ORDER) {
			recs[count].unknown=l_swap(recs[count].unknown);
			recs[count].offset=l_swap(recs[count].offset);
			recs[count].length=l_swap(recs[count].length);
		}
//		fprintf(stderr,"%08X %08X %08X\n",recs[count].offset,recs[count].length,recs[count].unknown);
		if( (recs[count].offset > 0x800000) || (recs[count].length>0xFFFF) || (!recs[count].length) || 
			(recs[count].length&1) ||((recs[count].length+recs[count].offset) > 0x800000))
			break;
		count++;
	}
	if(count>1) {
		recs[count].unknown=0;
		recs[count].offset=0;
		recs[count].length=0;
		return recs;
	}
	else {
		free(recs);
		return NULL;
	}
}

struct plugrecord *read_bin_plugs(int fd) {
	struct plugrecord *rrecs, *recs=NULL;
	int count=0;	
	
	while(1) {
		if(!(rrecs=realloc( recs, sizeof(struct plugrecord) * (++count)))) {
			free(recs);
			return NULL;
		}
		else
			recs=rrecs;
		if(read(fd,recs+(count-1), sizeof(struct plugrecord)) != sizeof(struct plugrecord))
			break;
		if(src_endian != __BYTE_ORDER) {
			recs[count-1].code=l_swap(recs[count-1].code);
			recs[count-1].offset=l_swap(recs[count-1].offset);
			recs[count-1].length=l_swap(recs[count-1].length);
		}
		if( ((recs[count-1].code&0xFF000000) != 0xFF000000) || (recs[count-1].length>=(0xFFFF<<1)) || 
			((recs[count-1].offset+recs[count-1].length)>=0x800000) )
			break;
	}
	if(count>1) {
		recs[count-1].code=0;
		recs[count-1].offset=0;
		recs[count-1].length=0;
		return recs;
	}
	else {
		free(recs);
		return NULL;
	}
}

int read_bin_content_from(int fd, off_t offset, struct content *cnt) {
	if(lseek(fd,offset,SEEK_SET)!=offset)
		return -1;
	else
		return read_bin_content(fd,cnt);
}

struct segmrecord *read_bin_segms_from(int fd, off_t offset) {
	if(lseek(fd,offset,SEEK_SET)!=offset)
		return NULL;
	else
		return read_bin_segms(fd);
}

struct plugrecord *read_bin_plugs_from(int fd, off_t offset) {
	if(lseek(fd,offset,SEEK_SET)!=offset)
		return NULL;
	else
		return read_bin_plugs(fd);
}

int write_asc_cnt_segms(FILE *fl, struct segmrecord *segms, struct content *cnts) {
	int count, count2=0;
	
	while(cnts[count2].length) {
		fprintf(fl, "SEG   %08lX %08lX %08lX\n",segms[count2].offset,segms[count2].length,segms[count2].unknown);
		for(count=0;count<cnts[count2].length;count+=2) {
			if(!(count&0x0F)) {
				if (count)
					fprintf(fl,"\n");
				fprintf(fl, "DATA  ");
			}
			fprintf(fl,"%02X%02X ",cnts[count2].data[count],cnts[count2].data[count+1]);
		}
		fprintf(fl,"\n");
		count2++;
	}
	return count2;
}